% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/portion.R
\name{portion}
\alias{portion}
\alias{portion_R}
\alias{portion_IR}
\alias{portion_I}
\alias{portion_SI}
\alias{portion_S}
\title{Calculate resistance of isolates}
\source{
\strong{M39 Analysis and Presentation of Cumulative Antimicrobial Susceptibility Test Data, 4th Edition}, 2014, \emph{Clinical and Laboratory Standards Institute (CLSI)}. \url{https://clsi.org/standards/products/microbiology/documents/m39/}.
}
\usage{
portion_R(ab1, ab2 = NULL, minimum = 30, as_percent = FALSE)

portion_IR(ab1, ab2 = NULL, minimum = 30, as_percent = FALSE)

portion_I(ab1, minimum = 30, as_percent = FALSE)

portion_SI(ab1, ab2 = NULL, minimum = 30, as_percent = FALSE)

portion_S(ab1, ab2 = NULL, minimum = 30, as_percent = FALSE)
}
\arguments{
\item{ab1}{vector of antibiotic interpretations, they will be transformed internally with \code{\link{as.rsi}} if needed}

\item{ab2}{like \code{ab}, a vector of antibiotic interpretations. Use this to calculate (the lack of) co-resistance: the probability where one of two drugs have a resistant or susceptible result. See Examples.}

\item{minimum}{minimal amount of available isolates. Any number lower than \code{minimum} will return \code{NA}. The default number of \code{30} isolates is advised by the CLSI as best practice, see Source.}

\item{as_percent}{logical to indicate whether the output must be returned as percent (text), will else be a double}
}
\value{
Double or, when \code{as_percent = TRUE}, a character.
}
\description{
These functions can be used to calculate the (co-)resistance of microbial isolates (i.e. percentage S, SI, I, IR or R). All functions can be used in \code{dplyr}s \code{\link[dplyr]{summarise}} and support grouped variables, see \emph{Examples}.

\code{portion_R} and \code{portion_IR} can be used to calculate resistance, \code{portion_S} and \code{portion_SI} can be used to calculate susceptibility.\cr
}
\details{
\strong{Remember that you should filter your table to let it contain only first isolates!} Use \code{\link{first_isolate}} to determine them in your data set.

The old \code{\link{rsi}} function is still available for backwards compatibility but is deprecated.
\if{html}{
  \cr\cr
  To calculate the probability (\emph{p}) of susceptibility of one antibiotic, we use this formula:
  \out{<div style="text-align: center">}\figure{mono_therapy.png}\out{</div>}
  To calculate the probability (\emph{p}) of susceptibility of more antibiotics (i.e. combination therapy), we need to check whether one of them has a susceptible result (as numerator) and count all cases where all antibiotics were tested (as denominator). \cr
  \cr
  For two antibiotics:
  \out{<div style="text-align: center">}\figure{combi_therapy_2.png}\out{</div>}
  \cr
  Theoretically for three antibiotics:
  \out{<div style="text-align: center">}\figure{combi_therapy_3.png}\out{</div>}
}
}
\examples{
# Calculate resistance
portion_R(septic_patients$amox)
portion_IR(septic_patients$amox)

# Or susceptibility
portion_S(septic_patients$amox)
portion_SI(septic_patients$amox)

# Since n_rsi counts available isolates (and is used as denominator),
# you can calculate back to count e.g. non-susceptible isolates:
portion_IR(septic_patients$amox) * n_rsi(septic_patients$amox)

library(dplyr)
septic_patients \%>\%
  group_by(hospital_id) \%>\%
  summarise(p = portion_S(cipr),
            n = n_rsi(cipr)) # n_rsi works like n_distinct in dplyr

septic_patients \%>\%
  group_by(hospital_id) \%>\%
  summarise(R = portion_R(cipr, as_percent = TRUE),
            I = portion_I(cipr, as_percent = TRUE),
            S = portion_S(cipr, as_percent = TRUE),
            n = n_rsi(cipr), # works like n_distinct in dplyr
            total = n())     # NOT the amount of tested isolates!

# Calculate co-resistance between amoxicillin/clav acid and gentamicin,
# so we can see that combination therapy does a lot more than mono therapy:
portion_S(septic_patients$amcl) # S = 67.3\%
n_rsi(septic_patients$amcl)     # n = 1570

portion_S(septic_patients$gent) # S = 74.0\%
n_rsi(septic_patients$gent)     # n = 1842

with(septic_patients,
     portion_S(amcl, gent))     # S = 92.1\%
with(septic_patients,           # n = 1504
     n_rsi(amcl, gent))

septic_patients \%>\%
  group_by(hospital_id) \%>\%
  summarise(cipro_p = portion_S(cipr, as_percent = TRUE),
            cipro_n = n_rsi(cipr),
            genta_p = portion_S(gent, as_percent = TRUE),
            genta_n = n_rsi(gent),
            combination_p = portion_S(cipr, gent, as_percent = TRUE),
            combination_n = n_rsi(cipr, gent))

\dontrun{

# calculate current empiric combination therapy of Helicobacter gastritis:
my_table \%>\%
  filter(first_isolate == TRUE,
         genus == "Helicobacter") \%>\%
  summarise(p = portion_S(amox, metr),  # amoxicillin with metronidazole
            n = n_rsi(amox, metr))
}
}
\seealso{
\code{\link{n_rsi}} to count cases with antimicrobial results.
}
\keyword{antibiotics}
\keyword{isolate}
\keyword{isolates}
\keyword{resistance}
\keyword{rsi}
\keyword{rsi_df}
\keyword{susceptibility}
